# Generated by Django 4.2.23 on 2025-10-14 11:53

import django.db.models.deletion
from django.db import migrations
from django.db import models


class Migration(migrations.Migration):

    dependencies = [
        ("contenttypes", "0002_remove_content_type_name"),
        ("custom_fields", "0002_make_custom_field_label_and_value_text_translateable"),
    ]

    operations = [
        migrations.AddField(
            model_name="customfield",
            name="type_content_type",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="customfield_set_for_type",
                to="contenttypes.contenttype",
            ),
        ),
        migrations.AddField(
            model_name="customfield",
            name="type_id",
            field=models.PositiveIntegerField(blank=True, null=True),
        ),
    ]
